package net.knarcraft.paidsigns.listener;

import net.knarcraft.paidsigns.PaidSigns;
import net.knarcraft.paidsigns.container.PaidSign;
import net.knarcraft.paidsigns.manager.EconomyManager;
import net.knarcraft.paidsigns.manager.PaidSignManager;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.SignChangeEvent;

import java.util.List;

/**
 * A listener for listening to registered paid signs
 */
public class SignListener implements Listener {

    @EventHandler(priority = EventPriority.MONITOR)
    public void onSignChange(SignChangeEvent event) {
        if (event.isCancelled() || event.getPlayer().hasPermission("paidsigns.paymentexempt")) {
            return;
        }

        String[] lines = event.getLines();
        PaidSignManager signManager = PaidSigns.getInstance().getSignManager();
        for (short lineIndex = 0; lineIndex < lines.length; lineIndex++) {
            //Get all "weak" matches (any paid sign with a clean id matching the clean line)
            List<PaidSign> matchingSigns = signManager.getPaidSigns(PaidSign.getCleanString(lines[lineIndex]), lineIndex);
            if (matchingSigns.isEmpty()) {
                continue;
            }
            if (testMatchingSigns(lineIndex, lines[lineIndex], matchingSigns, event)) {
                return;
            }
        }
    }

    /**
     * Tests all weak matches of paid signs to check if a strong match is found
     *
     * @param lineIndex     <p>The index of the currently managed line</p>
     * @param line          <p>The text on the currently managed line</p>
     * @param matchingSigns <p>The signs that weakly match the </p>
     * @param event         <p>The triggered sign change event</p>
     * @return <p>True if a match was found and thw work is finished</p>
     */
    private boolean testMatchingSigns(short lineIndex, String line, List<PaidSign> matchingSigns, SignChangeEvent event) {
        for (PaidSign paidSign : matchingSigns) {
            if (paidSign.matches(lineIndex, line)) {
                Player player = event.getPlayer();
                double cost = paidSign.getCost();
                boolean canAfford = EconomyManager.canAfford(player, cost);
                if (!canAfford) {
                    player.sendMessage("[PaidSigns] You cannot afford to create this sign");
                    event.setCancelled(true);
                } else {
                    String unit = EconomyManager.getCurrency(cost != 1);
                    player.sendMessage(String.format("[PaidSigns] You paid %.2f %s to create the sign", cost, unit));
                    EconomyManager.withdraw(player, cost);
                }
                return true;
            }
        }
        return false;
    }

}
